/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


#ifdef __GNUG__
#pragma interface
#endif

#ifndef __set_HXX
#define __set_HXX

#include "hem.hxx"
#include <sys/types.h>

#define SETROOT 256
#define SETNODE 4

#include "setident.hxx"

template <class ELEM> class SetCell {
  private :
  SetIdent _id ;

  public :
  SetCell(ELEM* el) : _id(el) {}
  SetIdent& ident() { return _id ; }
  ELEM* elem() { return (ELEM*)_id.adr() ; }
} ;


template <class ELEM>
class SetPtr : public HEM<SETROOT, SETNODE, SetCell<ELEM>, SetIdent> {
  private :

  int _size ;

  public :

  SetPtr() : _size(0) {} 
  ELEM* operator[](ELEM* el) {
    SetCell<ELEM>* cell ;
    cell = this->HEM<SETROOT,SETNODE,SetCell<ELEM>,SetIdent>::operator[](el);
    return cell != NULL ? cell->elem() : NULL ;
  }
  
  int member(ELEM* el) {
    return (*this)[el] != NULL ;
  }
  int operator+=(ELEM* el) {
    if (this->member(el)) return 0 ;
    _size++ ;
    SetCell<ELEM>* cell = new SetCell<ELEM>(el) ;
    this->HEM<SETROOT,SETNODE,SetCell<ELEM>,SetIdent>::operator+=(cell);
    return 1 ;
  }
  int operator-=(ELEM* el) {
    if (this->member(el) == 0) return 0 ;
    _size-- ;
    SetCell<ELEM>* cell ;
    cell = this->HEM<SETROOT,SETNODE,SetCell<ELEM>,SetIdent>::operator[](el) ;
    this->HEM<SETROOT,SETNODE,SetCell<ELEM>,SetIdent>::operator-=(el);
    delete cell ;
    return 1 ;
  }
  int count() { return _size ; }
  void clear() {
    _size = 0 ;
    IterHEM<SETROOT, SETNODE, SetCell<ELEM>, SetIdent> iter(*this) ;
    for (; iter.more(); delete iter.get(), iter.next()) ;
    this->HEM<SETROOT,SETNODE,SetCell<ELEM>,SetIdent>::clear() ;
  }
} ;


template <class ELEM>
class IterSetPtr : public IterHEM<SETROOT, SETNODE, SetCell<ELEM>, SetIdent> {
  public :

  IterSetPtr(SetPtr<ELEM>& s) : IterHEM<SETROOT, SETNODE, SetCell<ELEM>, SetIdent>(s) {} ;
  ELEM* get() {
   return this->IterHEM<SETROOT,SETNODE,SetCell<ELEM>,SetIdent>::get()->elem();
  }
} ;

#endif
